
/******************************************************************************
* MODULE     : base.hpp
* DESCRIPTION: Basic inclusions, definitions and routines
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_BASE_HPP
#define __MML_BASE_HPP
#include "config.hpp"

#ifndef MMX_GC
#define MMX_GC
#ifdef SUPPORTS_GC
#ifdef OLD_CPP
#include <gc.h>
inline void* operator new (std::size_t sz) { return GC_MALLOC (sz); }
inline void* operator new[] (std::size_t sz) { return GC_MALLOC (sz); }
inline void operator delete (void* dummy) { (void) dummy; }
inline void operator delete[] (void* dummy ) { (void) dummy; }
#else
#include <gc/new_gc_alloc.h>
#include <new>
inline void* operator new (std::size_t sz) throw (std::bad_alloc) {
  return GC_MALLOC (sz); }
inline void* operator new[] (std::size_t sz) throw (std::bad_alloc) {
  return GC_MALLOC (sz); }
inline void operator delete (void*) throw() {}
inline void operator delete[] (void*) throw() {}
inline void* operator new (std::size_t sz, const std::nothrow_t& d) throw() {
  (void) d; return GC_MALLOC (sz); }
inline void* operator new[] (std::size_t sz, const std::nothrow_t& d) throw() {
  (void) d; return GC_MALLOC (sz); }
inline void operator delete (void*, const std::nothrow_t& d) throw() {
  (void) d; }
inline void operator delete[] (void*, const std::nothrow_t& d) throw() {
  (void) d; }
#endif
#endif
#endif

#include <stdlib.h>
#include <iostream>
using std::istream;
using std::ostream;
using std::cin;
using std::cout;
using std::cerr;
namespace mml {

#define TYPE_PROTO(T,R)                                                 \
  class T;                                                              \
  class R;                                                              \
  typedef const T& T##_in;
  
#define INDIRECT_PROTO(T,R)                                             \
private:						                \
  R* rep;						                \
public:							                \
  inline T (R* rep2, bool old_flag);              		        \
  inline T (const T& x);                 		                \
  inline ~T ();   					                \
  inline T& operator= (const T& x);                              	\
  inline R* operator-> () const;

#ifdef SUPPORTS_GC // indirect structures for Boehm's GC
#define REP_STRUCT
#define INDIRECT_IMPL(T,R)                                              \
  inline T::T (R* rep2, bool f): rep (rep2) { (void) f; }   	        \
  inline T::T (const T& x) : rep (x.rep) {}		                \
  inline T::~T () {}					                \
  inline T& T::operator= (const T& x) { rep= x.rep; return *this; }	\
  inline R* T::operator-> () const { return rep; }                      \
  TYPE_PROTO(T,R)

#else // indirect structures through reference counting
struct rep_struct {
  int ref_count;
  inline rep_struct (): ref_count (1) {}
  virtual inline ~rep_struct () {}
};
#define REP_STRUCT :public rep_struct
#define INC_COUNT(x) { if ((x)!=NULL) (x)->ref_count++; }
#define DEC_COUNT(x) { if ((x)!=NULL && 0==--((x)->ref_count)) delete (x); }
#define INDIRECT_IMPL(T,R)                                              \
  inline T::T (R* rep2, bool inc): rep(rep2) {                          \
    if (inc) INC_COUNT (rep); }                                         \
  inline T::T (const T& x): rep(x.rep) { INC_COUNT (rep); }             \
  inline T::~T () { DEC_COUNT (rep); }                                  \
  inline R* T::operator -> () const { return rep; }                     \
  inline T& T::operator = (const T& x) {                                \
    INC_COUNT (x.rep); DEC_COUNT (rep);                                 \
    rep=x.rep; return *this; }                                          \
  TYPE_PROTO(T,R)
#endif

volatile void fatal_error (char* message, char* routine, char* file);
void mml_init ();

}
#endif // __MML_BASE_HPP
