
/******************************************************************************
* MODULE     : mml_gmp.cpp
* DESCRIPTION: Additional code for interface to gmp and mpfr
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "mml_gmp.hpp"
namespace mml {

mp_prec_t mml_default_prec = BITS_PER_LIMB<<1;
int       mml_default_round= GMP_RNDN;
  mp_prec_t mml_default_low  = BITS_PER_LIMB; // must be >= BITS_PER_LIMB

ostream&
print (ostream& out, const mpfr_t x, mp_prec_t bits) {
  if (mpfr_nan_p (x) != 0) return out << "NaN";
  mp_prec_t digs= bits==0? 0: max (((int) (0.301029995 * bits)) - 1, 2);
  mp_exp_t  exponent;
  char* mantissa_s= mpfr_get_str (NULL, &exponent, 10, digs, x, GMP_RNDN);
  int first= 0;
  if (mantissa_s[0] == '-') { out << "-"; first++; }
  out << mantissa_s[first];
  out << ".";
  out << (mantissa_s + (first + 1));
  out << "e" << (exponent-1);
  delete[] mantissa_s;
  return out;
}

ostream&
operator << (ostream& out, const mpfr_t x) {
  return print (out, x, 0);
}

} // namespace mml
