
/******************************************************************************
* MODULE     : interval_abs.cpp
* DESCRIPTION: Compute the absolute value interval
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
abs (interval_in x) {
  if (is_error (x)) return x;
  interval r= raw_interval (prec (x));
  if (has_low_prec (x)) {
    mpfr_abs (r->min, x->min, GMP_RNDD);
    mpfr_abs (r->max, x->max, GMP_RNDU);
    if (mpfr_cmp (r->min, r->max) > 0) {
      mpfr_abs (r->min, x->max, GMP_RNDD);
      mpfr_abs (r->max, x->min, GMP_RNDU);
    }
    if (mpfr_sgn (x->min) * mpfr_sgn (x->max) < 0)
      mpfr_set_si (r->min, 0, GMP_RNDN);
  }
  else {
    mpfr_abs (r->mid, x->mid, GMP_RNDN);
    mpfr_set (r->rad, x->rad, GMP_RNDU);
  }
  return r;
}

} // namespace mml
