
/******************************************************************************
* MODULE     : interval_reduce.cpp
* DESCRIPTION: Reduce the precision of an interval
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
reduce (interval_in x) {
  if (has_low_prec (x)) return x;
  else {
    interval r= raw_interval ();
    mpfr_sub (r->min, x->mid, x->rad, GMP_RNDD);
    mpfr_add (r->max, x->mid, x->rad, GMP_RNDU);
    return r;
  }
}

interval
reduce (interval_in x, mp_prec_t new_prec) {
  if (has_low_prec (x) || (new_prec >= prec (x))) return x;
  else if (new_prec <= DEFAULT_LOW) {
    interval r= raw_interval ();
    mpfr_sub (r->min, x->mid, x->rad, GMP_RNDD);
    mpfr_add (r->max, x->mid, x->rad, GMP_RNDU);
    return r;
  }
  else {
    interval r= raw_interval (new_prec);
    mpfr_set (r->mid, x->mid, GMP_RNDN);
    mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
    mpfr_add (r->rad, r->rad, x->rad, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
