
/******************************************************************************
* MODULE     : interval_sint.cpp
* DESCRIPTION: Construct an interval from a signed integer
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval_rep::interval_rep (signed long int i, mp_prec_t prec) {
  if (i == 0) {
    mpfr_init2 (min, DEFAULT_LOW);
    mpfr_init2 (max, DEFAULT_LOW);
    mpfr_set_si (max, 0, GMP_RNDU);
    mpfr_neg (min, max, GMP_RNDD);
  }
  else if (prec <= (DEFAULT_LOW + MIN_SECURITY)) {
    mpfr_init2 (min, DEFAULT_LOW);
    mpfr_init2 (max, DEFAULT_LOW);
    mpfr_set_si (min, i, GMP_RNDD);
    mpfr_set_si (max, i, GMP_RNDU);
    mpfr_nextbelow (min);
    mpfr_nextabove (max);
  }
  else {
    mpfr_init2 (mid, prec);
    mpfr_init2 (rad, DEFAULT_LOW);
    mpfr_set_si (mid, i, GMP_RNDN);
    mpfr_set_ui_2si (rad, 1, mpfr_get_step (mid), GMP_RNDU);
  }
}

} // namespace mml
